open Xlist

open Interfaces

(** class of grammar constructors *)
class constructor_impl _gmod _name _params : constructor =
object (self)
	(** grammar module of this element *)
	val mutable gmod : gmodule = _gmod
	
	(** constructor name *)
	val name : string = _name
	(** parameter list *)
	val params : parameter list = _params
	
	(** accessors *)
	method get_name : string = name
	method get_params : parameter list = params
	
	(** post-initialization with the given gmodule *)
	method set_gmodule (gm : gmodule) : unit =
		begin
			(* set the gmod field *)
			gmod <- gm;
			(* performs post_initialization for parameters *)
			List.iter (fun p -> p#set_gmodule gm) params
		end
	
	(** returns true/false whether this is an index constructor *)
	method is_index : bool =
		List.exists (fun p -> p#is_index) params
	
	(** returns the index name (the first parameter name) *)
	method get_index_name : string =
		match params with
			[] -> ""
		| p :: _ -> p#get_name
	
	(** returns the through list of the category of each parameter
	* along with the categories themselves if they have index *)
	method get_throughs : grammar list =
		Xlist.doublon_remove
			(List.concat
					(List.map (fun p -> p#get_throughs) params))
	
	(** induction pattern for this constructor *)
	method induction_pattern (g : grammar) : string list =
		List.concat (List.map (fun p -> p#induction_pattern g) params)
	
	(** rewrite induction hypothesis for this constructor *)
	method rewrite_induction (prop : property) (g : grammar) : string =
		let params_ind =
			List.map (fun p -> p#rewrite_induction prop g) params in
		let non_empty_params_ind =
			List.filter (fun s -> s <> "") params_ind in
		String.concat "; " non_empty_params_ind
	
	(** function equation generation *)
	method fun_equation_gen (def : fun_definition) : string list =
		def#constructor_function_equation_gen (self :> constructor)
	
	(** Pretty printing *)
	method to_string () : string =
		"  | " ^ name ^ (String.concat "" (List.map (fun p -> p#to_string ()) params))
end
