(**
 * Generic main structure
 *)

open Debug
open Version
open Dbgen

(* The minimum expected numbers of arguments on the command line *)
let number_param_min = 2
;;

let version =
    "DBgen " ^ Version.number ^ " (" ^ Version.date ^ ")"
;;

(**
 * displays the program command usage and exits.
 *)
let usage _ =
    (print_string ("usage: dbgen [ -version ][ -debug ] in-file out-file\n");
     exit Sys.sigabrt)
;;

(**
 * calls the start function of program with the arguments list.
 *)
let run _ =
  let _ = Debug.println "# run start." in
  let arg_list = List.tl (Array.to_list Sys.argv)
  in
  let real_arg_list = List.filter (fun s -> s.[0] <> '-') arg_list
  and _ =
    if List.mem "-version" arg_list
    then
      begin
	print_string (version ^ "\n"); exit 0
      end else ()
  in
  try
    let in_file = List.nth real_arg_list 0
    and out_file = List.nth real_arg_list 1 in
    DBgen.start in_file out_file
  with
    Failure s -> if s = "nth" then usage () else failwith s
;;

(**
 * calls run() and wait for any exception in order to display the exception message.
 *)
let main _ =
  let _ = Debug.println "# main start." in
  let _ =
(*    try*) run ()
(*    with e -> 
      begin
	print_string (Printexc.to_string e); print_newline ();
	exit 1
      end*)
  in
  exit 0
;;

let _ = Debug.println "#" in
main ()
;;
